@@ -29,7 +29,6 @@
 #include <errno.h>
 #include <X11/Xlib.h>
 #include <X11/Xmd.h>
-#include <X11/Xdmcp.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
@@ -40,9 +39,10 @@
 #include <fcntl.h>
 
 #include "gdm.h"
+#include "misc.h"
 #include "choose.h"
 
-static const gchar RCSid[]="$Id: choose.c,v 1.11 2001/07/02 21:10:54 jirka Exp $";
+static const gchar RCSid[]="$Id: choose.c,v 1.13 2001/07/05 10:27:29 jirka Exp $";
 
 static gint ipending = 0;
 static GSList *indirect = NULL;
@@ -51,8 +51,6 @@ static GSList *indirect = NULL;
 extern gint GdmMaxIndirect;	/* Maximum pending indirects, i.e. simultaneous choosing sessions */
 extern gint GdmMaxIndirectWait;	/* Maximum age before a pending session is removed from the list */
 
-extern void gdm_debug (gchar *, ...);
-
 static guint indirect_id = 1;
 
 static gboolean
@@ -81,34 +79,33 @@ remove_oldest_pending (void)
 }
 
 gboolean
-gdm_choose_socket_handler (GIOChannel *source, GIOCondition cond, gpointer data)
+gdm_choose_data (const char *data)
 {
-	gchar buf[PIPE_SIZE];
-	gint len;
 	int id;
 	struct in_addr addr;
 	GSList *li;
+	char *msg = g_strdup (data);
+	char *p;
 
-	if (cond != G_IO_IN) 
-		return TRUE;
-
-	if (g_io_channel_read (source, buf, PIPE_SIZE, &len) != G_IO_ERROR_NONE)
-		return TRUE;
-
-	gdm_debug ("gdm_choose_socket_handler: Read %d bytes", len);
-
-	if (buf[0] != STX)
-		return TRUE;
-
-	if (len != 1/*stx*/ + sizeof (int) /* id */ + sizeof (struct in_addr)) {
-		gdm_debug ("gdm_choose_socket_handler: Data not correct size");
-		return TRUE;
+	p = strtok (msg, " ");
+	if (p == NULL || strcmp (GDM_SOP_CHOSEN, p) != 0) {
+		g_free (msg);
+		return FALSE;
 	}
 
-	id = *(int *) (&buf[1]);
-	memcpy (&addr, &buf[1+sizeof(int)], sizeof (struct in_addr));
+	p = strtok (NULL, " ");
+	if (p == NULL || sscanf (p, "%d", &id) != 1) {
+		g_free (msg);
+		return FALSE;
+	}
 
-	gdm_debug ("gdm_choose_socket_handler: got indirect id: %d address: %s",
+	p = strtok (NULL, " ");
+	if (p == NULL || inet_aton (p, &addr) == 0) {
+		g_free (msg);
+		return FALSE;
+	}
+
+	gdm_debug ("gdm_choose_data: got indirect id: %d address: %s",
 		   id, inet_ntoa (addr));
 
 	for (li = indirect; li != NULL; li = li->next) {
@@ -132,7 +129,7 @@ gdm_choose_socket_handler (GIOChannel *source, GIOCondition cond, gpointer data)
 		}
 	}
 
-	return TRUE;
+	return FALSE;
 }
 
 
@@ -234,7 +231,7 @@ gdm_choose_indirect_dispose (GdmIndirectDisplay *id)
 	    ipending--;
     id->acctime = 0;
 
-    gdm_debug ("gdm_choose_indirect_dispose: Disposing %d", 
+    gdm_debug ("gdm_choose_indirect_dispose: Disposing %s", 
 	       inet_ntoa (id->dsp_sa->sin_addr));
 
     g_free (id->dsp_sa);
