@@ -28,11 +28,7 @@ void			gdm_choose_indirect_dispose (GdmIndirectDisplay *id);
 /* dispose of indirect display of id, if no host is set */
 void			gdm_choose_indirect_dispose_empty_id (guint id);
 
-gboolean		gdm_choose_socket_handler (GIOChannel *source,
-						   GIOCondition cond,
-						   gpointer data);
-
-
+gboolean		gdm_choose_data (const char *data);
 
 #endif /* CHOOSE_H */
 
