@@ -49,7 +49,7 @@
 #include "misc.h"
 #include "gdmwm.h"
 
-static const gchar RCSid[]="$Id: gdmchooser.c,v 1.25 2001/07/02 21:10:54 jirka Exp $";
+static const gchar RCSid[]="$Id: gdmchooser.c,v 1.26 2001/07/05 10:27:30 jirka Exp $";
 
 /* set the DOING_GDM_DEVELOPMENT env variable if you want to
  * search for the glade file in the current dir and not the system
@@ -73,7 +73,7 @@ static gboolean gdm_chooser_decode_packet (GIOChannel   *source,
 					   GIOCondition  condition,
 					   gpointer      data);
 
-static void gdm_chooser_abort (const gchar *format, ...);
+static void gdm_chooser_abort (const gchar *format, ...) G_GNUC_PRINTF (1, 2);
 static void gdm_chooser_browser_update (void);
 static void gdm_chooser_xdmcp_init (char **hosts);
 static void gdm_chooser_host_dispose (GdmChooserHost *host);
